(*----------------------------------------------------------------------------*
 *  Direct3D sample based on DirectX 8.0 SDK Text3D example                   *
 *  by Alexey Barkovoy                                                        *
 *  E-Mail: clootie@reactor.ru                                                *
 *                                                                            *
 *  This example demonstrates use of TD3DTextureFont class whitch has syntax  *
 *  very close to CD3DFont from DirectX 8 SDK samples framework               *
 *  Sample itself is modified version if Text3D DirectX SDK sample - this can *
 *  really help you to switch using it from CD3DFont class.                   *
 *                                                                            *
 *  Modified: 10-Nov-2001                                                     *
 *                                                                            *
 *  Latest version can be downloaded from:                                    *
 *     http://clootie.narod.ru                                                *
 *----------------------------------------------------------------------------*)

unit BitmapFontUnit;

interface

uses
  Windows, Messages, SysUtils, CommDlg,
  D3DX8, D3DApp, D3DFont, D3DFile, D3DUtil, DXUtil, D3DTextureFont,
  {$IFDEF DXG_COMPAT} DirectXGraphics {$ELSE} Direct3D8 {$ENDIF};

//-----------------------------------------------------------------------------
// Defines, constants, and global variables
//-----------------------------------------------------------------------------

type
  // Custom D3D vertex format used by the vertex buffer
  PMyVertex = ^TMyVertex;
  TMyVertex = packed record
    p: TD3DXVector3;       // vertex position
    n: TD3DXVector3;       // vertex normal
  end;

  PMyVertexArray = ^TMyVertexArray;
  TMyVertexArray = array [0..MaxInt div SizeOf(TMyVertex) - 1] of TMyVertex;

const
  D3DFVF_MYVERTEX = (D3DFVF_XYZ or D3DFVF_NORMAL);

type
//-----------------------------------------------------------------------------
// Name: class CMyD3DApplication
// Desc: Application class. The base class (CD3DApplication) provides the 
//       generic functionality needed in all Direct3D samples. CMyD3DApplication
//       adds functionality specific to this sample program.
//-----------------------------------------------------------------------------
  CMyD3DApplication = class(CD3DApplication)
    // m_pFont: CD3DFont;
    m_pFont: TD3DTextureFont;
    m_pStatsFont: CD3DFont;
    m_strFont: array[0..99] of Char;
    m_dwFontSize: DWORD;

    m_matObj1: TD3DXMatrix;
    m_matObj2: TD3DXMatrix;
    m_matObj3: TD3DXMatrix;
    m_matObj4: TD3DXMatrix;
    m_matObj5: TD3DXMatrix;

  protected
    function OneTimeSceneInit: HRESULT; override;
    function InitDeviceObjects: HRESULT; override;
    function RestoreDeviceObjects: HRESULT; override;
    function InvalidateDeviceObjects: HRESULT;override;
    function DeleteDeviceObjects: HRESULT; override;
    function Render: HRESULT; override;
    function FrameMove: HRESULT; override;
    function FinalCleanup: HRESULT; override;

  public
    // LRESULT MsgProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam );
    constructor Create;
  end;

implementation

constructor CMyD3DApplication.Create;
begin
  inherited Create(0);

  m_strWindowTitle    := 'Text3D: Text in a 3D scene';
  m_bUseDepthBuffer   := False;

  // Create fonts
  m_strFont:= 'Arial';
  m_dwFontSize := 18;
//  m_pFont      := CD3DFont.Create(m_strFont, m_dwFontSize);
  m_pFont      := TD3DTextureFont.Create('CustomFont2.bmp', $FF00FF00);
  m_pStatsFont := CD3DFont.Create('Arial', 12, D3DFONT_BOLD);
end;

//-----------------------------------------------------------------------------
// Name: OneTimeSceneInit()
// Desc: Called during initial app startup, this function performs all the
//       permanent initialization.
//-----------------------------------------------------------------------------
function CMyD3DApplication.OneTimeSceneInit: HRESULT;
begin
  Result:= S_OK;
end;

//-----------------------------------------------------------------------------
// Name: FrameMove()
// Desc: Called once per frame, the call is the entry point for animating
//       the scene.
//-----------------------------------------------------------------------------
function CMyD3DApplication.FrameMove: HRESULT;
begin
  // Setup five rotation matrices (for rotating text strings)
  D3DXMatrixRotationAxis(m_matObj1, D3DXVector3(1.0,2.0,0.0), m_fTime/1.1);
  D3DXMatrixRotationAxis(m_matObj2, D3DXVector3(2.0,1.0,0.0), m_fTime/1.2);
  D3DXMatrixRotationAxis(m_matObj3, D3DXVector3(1.0,0.0,2.0), m_fTime/1.3);
  D3DXMatrixRotationAxis(m_matObj4, D3DXVector3(2.0,0.0,1.0), m_fTime/1.4);
  D3DXMatrixRotationAxis(m_matObj5, D3DXVector3(0.0,1.0,2.0), m_fTime/1.5);

  // Add some translational values to the matrices
  m_matObj1._41 := 1.0;   m_matObj1._42 := 3.0;   m_matObj1._43 := 0.0;
  m_matObj2._41 := 2.0;   m_matObj2._42 :=-2.0;   m_matObj2._43 := 0.0;
  m_matObj3._41 := 3.0;   m_matObj3._42 := 1.0;   m_matObj3._43 := 0.0;
  m_matObj4._41 :=-1.0;   m_matObj4._42 :=-3.0;   m_matObj4._43 := 0.0;
  m_matObj5._41 :=-2.0;   m_matObj5._42 := 2.0;   m_matObj5._43 := 0.0;

  Result:= S_OK;
end;

//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Called once per frame, the call is the entry point for 3d
//       rendering. This function sets up render states, clears the
//       viewport, and renders the scene.
//-----------------------------------------------------------------------------
function CMyD3DApplication.Render: HRESULT;
var
  mtrl: TD3DMaterial8;
begin
  // Clear the viewport
  m_pd3dDevice.Clear(0, nil, D3DCLEAR_TARGET, $00000000, 1.0, 0);

  // Begin the scene
  if SUCCEEDED(m_pd3dDevice.BeginScene) then
  begin
    // Draw the 5 spinning, colored 3D text objects
    D3DUtil_InitMaterial(mtrl, 1.0, 0.0, 0.0);
    m_pd3dDevice.SetMaterial(mtrl);
    m_pd3dDevice.SetTransform(D3DTS_WORLD, m_matObj1);
    // m_pFont.Render3DText('Red Text', D3DFONT_CENTERED or D3DFONT_TWOSIDED or D3DFONT_FILTERED);
    m_pFont.Render3DText('Red Text', [rfCentered, rfTwoSided, rfFiltered]);

    D3DUtil_InitMaterial(mtrl, 0.0, 1.0, 0.0);
    m_pd3dDevice.SetMaterial(mtrl);
    m_pd3dDevice.SetTransform(D3DTS_WORLD, m_matObj2);
    // m_pFont.Render3DText('Green Text', D3DFONT_CENTERED or D3DFONT_TWOSIDED or D3DFONT_FILTERED);
    m_pFont.Render3DText('Green Text', [rfCentered, rfTwoSided, rfFiltered]);

    D3DUtil_InitMaterial(mtrl, 0.0, 0.0, 1.0);
    m_pd3dDevice.SetMaterial(mtrl);
    m_pd3dDevice.SetTransform(D3DTS_WORLD, m_matObj3);
    // m_pFont.Render3DText('Blue Text', D3DFONT_CENTERED or D3DFONT_TWOSIDED or D3DFONT_FILTERED);
    m_pFont.Render3DText('Blue Text', [rfCentered, rfTwoSided, rfFiltered]);

    D3DUtil_InitMaterial(mtrl, 1.0, 0.0, 1.0);
    m_pd3dDevice.SetMaterial(mtrl);
    m_pd3dDevice.SetTransform(D3DTS_WORLD, m_matObj4);
    // m_pFont.Render3DText('Purple Text', D3DFONT_CENTERED or D3DFONT_TWOSIDED or D3DFONT_FILTERED);
    m_pFont.Render3DText('Purple Text', [rfCentered, rfTwoSided, rfFiltered]);

    D3DUtil_InitMaterial(mtrl, 1.0, 1.0, 0.0);
    m_pd3dDevice.SetMaterial(mtrl);
    m_pd3dDevice.SetTransform(D3DTS_WORLD, m_matObj5);
    // m_pFont.Render3DText('Yellow Text', D3DFONT_CENTERED or D3DFONT_TWOSIDED or D3DFONT_FILTERED);
    m_pFont.Render3DText('Yellow Text', [rfCentered, rfTwoSided, rfFiltered]);

    // Draw some 2D text
    m_pFont.DrawText(2, 40, $ffffffff, '2D Text');

    // Show frame rate
    m_pStatsFont.DrawText(2,  0, D3DCOLOR_ARGB(255,255,255,0), m_strFrameStats);
    m_pStatsFont.DrawText(2, 20, D3DCOLOR_ARGB(255,255,255,0), m_strDeviceStats);

    // End the scene.
    m_pd3dDevice.EndScene;
  end;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: InitDeviceObjects()
// Desc: Initialize scene objects.
//-----------------------------------------------------------------------------
function CMyD3DApplication.InitDeviceObjects: HRESULT;
begin
  // Restore the fonts
  m_pFont.InitDeviceObjects(m_pd3dDevice);
  m_pStatsFont.InitDeviceObjects(m_pd3dDevice);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: RestoreDeviceObjects()
// Desc: Initialize scene objects.
//-----------------------------------------------------------------------------
function CMyD3DApplication.RestoreDeviceObjects: HRESULT;
var
  vEyePt: TD3DXVector3;
  vLookatPt: TD3DXVector3;
  vUpVec: TD3DXVector3;
  matWorld, matView, matProj: TD3DXMatrix;
  fAspect: Single;
begin
  // Restore the fonts
  m_pFont.RestoreDeviceObjects;
  m_pStatsFont.RestoreDeviceObjects;

  // Restore the textures
  m_pd3dDevice.SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
  m_pd3dDevice.SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
  m_pd3dDevice.SetTextureStageState(0, D3DTSS_COLOROP,   D3DTOP_MODULATE);
  m_pd3dDevice.SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
  m_pd3dDevice.SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
  m_pd3dDevice.SetRenderState(D3DRS_DITHERENABLE, 1);
  m_pd3dDevice.SetRenderState(D3DRS_SPECULARENABLE, 1);
  m_pd3dDevice.SetRenderState(D3DRS_AMBIENT,  $00ffffff);

  // Set the transform matrices
  vEyePt    := D3DXVector3(0.0,-5.0,-10.0);
  vLookatPt := D3DXVector3(0.0, 0.0,  0.0);
  vUpVec    := D3DXVector3(0.0, 1.0,  0.0);

  D3DXMatrixIdentity(matWorld);
  D3DXMatrixLookAtLH(matView, vEyePt, vLookatPt, vUpVec);
  fAspect := m_d3dsdBackBuffer.Width / m_d3dsdBackBuffer.Height;
  D3DXMatrixPerspectiveFovLH(matProj, D3DX_PI/4, fAspect, 1.0, 100.0);

  m_pd3dDevice.SetTransform(D3DTS_WORLD,      matWorld);
  m_pd3dDevice.SetTransform(D3DTS_VIEW,       matView);
  m_pd3dDevice.SetTransform(D3DTS_PROJECTION, matProj);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: InvalidateDeviceObjects()
// Desc: Called when the app is exiting, or the device is being changed,
//       this function deletes any device dependent objects.
//-----------------------------------------------------------------------------
function CMyD3DApplication.InvalidateDeviceObjects: HRESULT;
begin
  m_pFont.InvalidateDeviceObjects;
  m_pStatsFont.InvalidateDeviceObjects;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: DeleteDeviceObjects()
// Desc: Called when the app is exiting, or the device is being changed,
//       this function deletes any device dependent objects.
//-----------------------------------------------------------------------------
function CMyD3DApplication.DeleteDeviceObjects: HRESULT;
begin
  m_pFont.DeleteDeviceObjects;
  m_pStatsFont.DeleteDeviceObjects;
  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: FinalCleanup()
// Desc: Called before the app exits, this function gives the app the chance
//       to cleanup after itself.
//-----------------------------------------------------------------------------
function CMyD3DApplication.FinalCleanup: HRESULT;
begin
  SAFE_DELETE(m_pFont);
  SAFE_DELETE(m_pStatsFont);

  Result:= S_OK;
end;

end.

